@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -17,6 +17,7 @@
 #include "gui/GOGUIPanel.h"
 #include "gui/GOGUIPanelView.h"
 #include "midi/GOMidiEvent.h"
+#include "size/GOResizable.h"
 #include "sound/GOSound.h"
 #include "threading/GOMutexLocker.h"
 
@@ -24,7 +25,6 @@
 #include "GOFrame.h"
 #include "GOOrgan.h"
 #include "GOOrganController.h"
-#include "GOResizable.h"
 #include "go_ids.h"
 
 GODocument::GODocument(GOResizable *pMainWindow, GOSound *sound)
@@ -46,13 +46,16 @@ bool GODocument::IsModified() const {
 }
 
 bool GODocument::LoadOrgan(
-  GOProgressDialog *dlg, const GOOrgan &organ, const wxString &cmb) {
+  GOProgressDialog *dlg,
+  const GOOrgan &organ,
+  const wxString &cmb,
+  bool isGuiOnly) {
   wxBusyCursor busy;
   GOConfig &cfg = m_sound.GetSettings();
 
   CloseOrgan();
   m_OrganController = new GOOrganController(cfg, this);
-  wxString error = m_OrganController->Load(dlg, organ, cmb);
+  wxString error = m_OrganController->Load(dlg, organ, cmb, isGuiOnly);
   if (!error.IsEmpty()) {
     if (error != wxT("!")) {
       wxLogError(wxT("%s\n"), error.c_str());
@@ -61,7 +64,8 @@ bool GODocument::LoadOrgan(
     CloseOrgan();
     return false;
   }
-  m_sound.GetSettings().AddOrgan(m_OrganController->GetOrganInfo());
+  cfg.AddOrgan(m_OrganController->GetOrganInfo());
+  cfg.Flush();
   {
     wxCommandEvent event(wxEVT_SETVALUE, ID_METER_AUDIO_SPIN);
     event.SetInt(m_OrganController->GetVolume());
@@ -70,16 +74,6 @@ bool GODocument::LoadOrgan(
     m_sound.GetEngine().SetVolume(m_OrganController->GetVolume());
   }
 
-  // synchronize cfg.ReleaseTail with OrganReleaseTail.
-  unsigned cfgReleaseTail = cfg.ReleaseLength();
-  unsigned organReleaseTail = m_OrganController->GetReleaseTail();
-
-  if (organReleaseTail) // organReleaseTail has the priority
-    cfg.ReleaseLength(organReleaseTail);
-  else if (cfgReleaseTail)
-    m_OrganController->SetReleaseTail(cfgReleaseTail);
-  cfg.Flush();
-
   wxCommandEvent event(wxEVT_WINTITLE, 0);
   event.SetString(m_OrganController->GetChurchName());
   wxTheApp->GetTopWindow()->GetEventHandler()->AddPendingEvent(event);
