@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -58,7 +58,10 @@ public:
   bool Save();
   bool Export(const wxString &cmb);
   bool LoadOrgan(
-    GOProgressDialog *dlg, const GOOrgan &organ, const wxString &cmb);
+    GOProgressDialog *dlg,
+    const GOOrgan &organ,
+    const wxString &cmb,
+    bool isGuiOnly);
   bool UpdateCache(GOProgressDialog *dlg, bool compress);
 
   void ShowMIDIEventDialog(
