@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -154,6 +154,8 @@ const wxString GOSetter::KEY_LOAD_FILE = wxT("LoadFile");
 const wxString GOSetter::KEY_SAVE_FILE = wxT("SaveFile");
 const wxString GOSetter::KEY_SAVE_SETTINGS = wxT("Save");
 
+const wxString GOSetter::KEY_ON_STATE = wxT("OnState");
+
 const wxString GOSetter::GROUP_REFRESH_FILES = wxT("SetterRefreshFiles");
 const wxString GOSetter::GROUP_PREV_FILE = wxT("SetterPrevFile");
 const wxString GOSetter::GROUP_CURR_FILE_NAME = wxT("SetterCurrFileName");
@@ -263,7 +265,7 @@ const struct GOElementCreator::ButtonDefinitionEntry GOSetter::m_element_types[]
     {wxT("TransposeUp"), ID_SETTER_TRANSPOSE_UP, true, true, false},
 
     {KEY_SAVE_SETTINGS, ID_SETTER_SAVE_SETTINGS, true, true, false},
-    {wxT("OnState"), ID_SETTER_ON, false, true, false},
+    {KEY_ON_STATE, ID_SETTER_ON, false, true, false},
 
     {wxT("CrescendoA"), ID_SETTER_CRESCENDO_A, true, true, false},
     {wxT("CrescendoB"), ID_SETTER_CRESCENDO_B, true, true, false},
@@ -1013,46 +1015,19 @@ void GOSetter::PushGeneral(
 }
 
 void GOSetter::PushDivisional(
-  GODivisionalCombination &cmb, GOButtonControl *pButtonToLight) {
-  GOCombination::ExtraElementsSet elementSet;
-  const GOCombination::ExtraElementsSet *pExtraSet
-    = GetCrescendoAddSet(elementSet);
+  GODivisionalCombination &cmb,
+  unsigned startManual,
+  unsigned cmbManual,
+  GOButtonControl *pButtonToLight) {
+  if (cmbManual == startManual || !m_state.m_IsActive) {
+    GOCombination::ExtraElementsSet elementSet;
+    const GOCombination::ExtraElementsSet *pExtraSet
+      = GetCrescendoAddSet(elementSet);
 
-  NotifyCmbPushed(cmb.Push(m_state, pExtraSet));
-  /* only use divisional couples, if not in setter mode */
-  if (!m_state.m_IsActive) {
-    unsigned cmbManualNumber = cmb.GetManualNumber();
-    unsigned divisionalNumber = cmb.GetDivisionalNumber();
-
-    for (unsigned k = 0; k < m_OrganController->GetDivisionalCouplerCount();
-         k++) {
-      GODivisionalCoupler *coupler = m_OrganController->GetDivisionalCoupler(k);
-      if (!coupler->IsEngaged())
-        continue;
-
-      for (unsigned i = 0; i < coupler->GetNumberOfManuals(); i++) {
-        if (coupler->GetManual(i) != cmbManualNumber)
-          continue;
-
-        for (unsigned int j = i + 1; j < coupler->GetNumberOfManuals(); j++)
-          m_OrganController->GetManual(coupler->GetManual(j))
-            ->GetDivisional(divisionalNumber)
-            ->Push();
-
-        if (coupler->IsBidirectional())
-          for (unsigned j = 0; j < coupler->GetNumberOfManuals(); j++) {
-            if (coupler->GetManual(j) == cmbManualNumber)
-              break;
-            m_OrganController->GetManual(coupler->GetManual(j))
-              ->GetDivisional(divisionalNumber)
-              ->Push();
-          }
-        break;
-      }
-    }
+    NotifyCmbPushed(cmb.Push(m_state, pExtraSet));
+    if (!pExtraSet)
+      UpdateAllSetsButtonsLight(pButtonToLight, cmbManual);
   }
-  if (!pExtraSet)
-    UpdateAllSetsButtonsLight(pButtonToLight, cmb.GetManualNumber());
 }
 
 void GOSetter::Next() { SetPosition(m_pos + 1); }
@@ -1188,7 +1163,8 @@ void GOSetter::SetTranspose(int value) {
 }
 
 void GOSetter::UpdateModified(bool modified) {
-  m_buttons[ID_SETTER_SAVE_SETTINGS]->Display(modified);
+  if (m_buttons.size())
+    m_buttons[ID_SETTER_SAVE_SETTINGS]->Display(modified);
 }
 
 GOEnclosure *GOSetter::GetEnclosure(const wxString &name, bool is_panel) {
