@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -194,7 +194,6 @@ GOConfig::GOConfig(wxString instance)
     PolyphonyLimit(
       this, wxT("General"), wxT("PolyphonyLimit"), 0, MAX_POLYPHONY, 2048),
     Preset(this, wxT("General"), wxT("Preset"), 0, MAX_PRESET, 0),
-    ReleaseLength(this, wxT("General"), wxT("ReleaseLength"), 0, 3000, 0),
     LanguageCode(this, wxT("General"), wxT("Language"), wxEmptyString),
     BitsPerSample(this, wxT("General"), wxT("BitsPerSample"), 8, 24, 24),
     Transpose(this, wxT("General"), wxT("Transpose"), -11, 11, 0),
@@ -215,6 +214,8 @@ GOConfig::GOConfig(wxString instance)
     MidiRecorderPath(
       this, wxT("General"), wxT("MIDIRecorderPath"), wxEmptyString),
     MidiPlayerPath(this, wxT("General"), wxT("MIDIPlayerPath"), wxEmptyString),
+    CheckForUpdatesAtStartup(
+      this, wxT("General"), wxT("CheckForUpdatesAtStartup"), true),
     m_MidiIn(MIDI_IN),
     m_MidiOut(MIDI_OUT) {
   m_ConfigFileName = GOStdPath::GetConfigDir() + wxFileName::GetPathSeparator()
