@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -64,6 +64,10 @@ GOSettingsOptions::GOSettingsOptions(GOConfig &settings, wxWindow *parent)
     wxLocale::GetLanguageName(wxLANGUAGE_GERMAN),
     new wxStringClientData(
       wxLocale::GetLanguageCanonicalName(wxLANGUAGE_GERMAN)));
+  m_Language->Append(
+    wxLocale::GetLanguageName(wxLANGUAGE_HUNGARIAN),
+    new wxStringClientData(
+      wxLocale::GetLanguageCanonicalName(wxLANGUAGE_HUNGARIAN)));
   m_Language->Append(
     wxLocale::GetLanguageName(wxLANGUAGE_ITALIAN),
     new wxStringClientData(
@@ -403,6 +407,16 @@ GOSettingsOptions::GOSettingsOptions(GOConfig &settings, wxWindow *parent)
   item6->Add(grid, 0, wxEXPAND | wxALL, 5);
   item9->Add(item6, 0, wxEXPAND | wxALL, 5);
 
+  item6 = new wxStaticBoxSizer(wxVERTICAL, this, _("&Updates"));
+  item9->Add(item6, 0, wxEXPAND | wxALL, 5);
+  item6->Add(
+    m_CheckForUpdatesAtStartup
+    = new wxCheckBox(this, wxID_ANY, _("Check for updates at startup")),
+    0,
+    wxEXPAND | wxALL,
+    5);
+  m_CheckForUpdatesAtStartup->SetValue(m_config.CheckForUpdatesAtStartup());
+
   item0->Add(item9, 1, wxEXPAND | wxALL, 0);
 
   topSizer->Add(item0, 0, wxEXPAND | wxALL, 5);
@@ -445,6 +459,7 @@ bool GOSettingsOptions::TransferDataFromWindow() {
   m_config.MemoryLimit(m_MemoryLimit->GetValue());
   m_config.MetronomeBPM(m_MetronomeBPM->GetValue());
   m_config.MetronomeMeasure(m_MetronomeMeasure->GetValue());
+  m_config.CheckForUpdatesAtStartup(m_CheckForUpdatesAtStartup->GetValue());
 
   // Language
   const wxStringClientData *const langData
