@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -42,26 +42,38 @@ void GOCoupler::PreparePlayback() {
   GODrawstop::PreparePlayback();
 
   GOManual *src = r_OrganModel.GetManual(m_SourceManual);
-  GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
 
   m_KeyVelocity.resize(src->GetLogicalKeyCount());
   std::fill(m_KeyVelocity.begin(), m_KeyVelocity.end(), 0);
-  m_InternalVelocity.resize(dest->GetLogicalKeyCount());
-  std::fill(m_InternalVelocity.begin(), m_InternalVelocity.end(), 0);
-  m_OutVelocity.resize(dest->GetLogicalKeyCount());
-  std::fill(m_OutVelocity.begin(), m_OutVelocity.end(), 0);
 
-  m_Keyshift = m_DestinationKeyshift + src->GetFirstLogicalKeyMIDINoteNumber()
-    - dest->GetFirstLogicalKeyMIDINoteNumber();
   if (m_FirstMidiNote > src->GetFirstLogicalKeyMIDINoteNumber())
     m_FirstLogicalKey
       = m_FirstMidiNote - src->GetFirstLogicalKeyMIDINoteNumber();
   else
     m_FirstLogicalKey = 0;
+  if (!m_UnisonOff) {
+    GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
+
+    m_InternalVelocity.resize(dest->GetLogicalKeyCount());
+    std::fill(m_InternalVelocity.begin(), m_InternalVelocity.end(), 0);
+    m_OutVelocity.resize(dest->GetLogicalKeyCount());
+    std::fill(m_OutVelocity.begin(), m_OutVelocity.end(), 0);
+
+    m_Keyshift = m_DestinationKeyshift + src->GetFirstLogicalKeyMIDINoteNumber()
+      - dest->GetFirstLogicalKeyMIDINoteNumber();
+  }
   if (m_UnisonOff && IsActive())
     src->SetUnisonOff(true);
 }
 
+void GOCoupler::SetRecursive(bool isRecursive) {
+  m_CoupleToSubsequentUnisonIntermanualCouplers = isRecursive;
+  m_CoupleToSubsequentUpwardIntermanualCouplers = isRecursive;
+  m_CoupleToSubsequentDownwardIntermanualCouplers = isRecursive;
+  m_CoupleToSubsequentUpwardIntramanualCouplers = isRecursive;
+  m_CoupleToSubsequentDownwardIntramanualCouplers = isRecursive;
+}
+
 const struct IniFileEnumEntry GOCoupler::m_coupler_types[] = {
   {wxT("Normal"), COUPLER_NORMAL},
   {wxT("Bass"), COUPLER_BASS},
@@ -80,11 +92,7 @@ void GOCoupler::Init(
   m_UnisonOff = unison_off;
   m_DestinationManual = dest_manual;
   m_DestinationKeyshift = keyshift;
-  m_CoupleToSubsequentUnisonIntermanualCouplers = recursive;
-  m_CoupleToSubsequentUpwardIntermanualCouplers = recursive;
-  m_CoupleToSubsequentDownwardIntermanualCouplers = recursive;
-  m_CoupleToSubsequentUpwardIntramanualCouplers = recursive;
-  m_CoupleToSubsequentDownwardIntramanualCouplers = recursive;
+  SetRecursive(recursive);
   GODrawstop::Init(cfg, group, name);
 
   m_CouplerType = coupler_type;
@@ -110,11 +118,7 @@ void GOCoupler::Load(GOConfigReader &cfg, wxString group) {
   m_DestinationKeyshift = cfg.ReadInteger(
     ODFSetting, group, wxT("DestinationKeyshift"), -24, 24, !m_UnisonOff, 0);
   if (m_UnisonOff) {
-    m_CoupleToSubsequentUnisonIntermanualCouplers = false;
-    m_CoupleToSubsequentUpwardIntermanualCouplers = false;
-    m_CoupleToSubsequentDownwardIntermanualCouplers = false;
-    m_CoupleToSubsequentUpwardIntramanualCouplers = false;
-    m_CoupleToSubsequentDownwardIntramanualCouplers = false;
+    SetRecursive(false);
     m_CouplerType = COUPLER_NORMAL;
     m_FirstMidiNote = 0;
     m_NumberOfKeys = 127;
@@ -164,11 +168,7 @@ void GOCoupler::Load(GOConfigReader &cfg, wxString group) {
       false,
       COUPLER_NORMAL);
     if (m_CouplerType == COUPLER_BASS || m_CouplerType == COUPLER_MELODY) {
-      m_CoupleToSubsequentUnisonIntermanualCouplers = false;
-      m_CoupleToSubsequentUpwardIntermanualCouplers = false;
-      m_CoupleToSubsequentDownwardIntermanualCouplers = false;
-      m_CoupleToSubsequentUpwardIntramanualCouplers = false;
-      m_CoupleToSubsequentDownwardIntramanualCouplers = false;
+      SetRecursive(false);
       if (!r_OrganModel.GetConfig().ODFCheck()) {
         cfg.ReadBoolean(
           ODFSetting,
@@ -253,6 +253,8 @@ void GOCoupler::SetupIsToStoreInCmb() {
 }
 
 void GOCoupler::SetOut(int noteNumber, unsigned velocity) {
+  assert(!m_UnisonOff); // ChangeKey doesn't call SetOut for UnisonOff
+
   if (noteNumber < 0)
     return;
   unsigned note = noteNumber;
@@ -267,9 +269,11 @@ void GOCoupler::SetOut(int noteNumber, unsigned velocity) {
   unsigned newstate = m_InternalVelocity[note];
   if (newstate)
     newstate--;
-  GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
   m_OutVelocity[note] = newstate;
-  dest->SetKey(note, m_OutVelocity[note], this, m_CouplerIndexInDest);
+
+  GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
+
+  dest->SetKey(note, m_OutVelocity[note], m_CouplerIndexInDest);
 }
 
 unsigned GOCoupler::GetInternalState(int noteNumber) {
@@ -354,24 +358,38 @@ void GOCoupler::SetKey(
 }
 
 void GOCoupler::ChangeState(bool on) {
-  GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
-
-  if (m_UnisonOff) {
+  if (m_UnisonOff)
     r_OrganModel.GetManual(m_SourceManual)->SetUnisonOff(on);
-    return;
-  }
+  else {
+    GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
 
-  for (unsigned i = 0; i < m_InternalVelocity.size(); i++) {
-    unsigned newstate = on ? m_InternalVelocity[i] : 0;
-    if (newstate > 0)
-      newstate--;
-    if (m_OutVelocity[i] != newstate) {
-      m_OutVelocity[i] = newstate;
-      dest->SetKey(i, m_OutVelocity[i], this, m_CouplerIndexInDest);
+    for (unsigned i = 0; i < m_InternalVelocity.size(); i++) {
+      unsigned newstate = on ? m_InternalVelocity[i] : 0;
+      if (newstate > 0)
+        newstate--;
+      if (m_OutVelocity[i] != newstate) {
+        m_OutVelocity[i] = newstate;
+        dest->SetKey(i, m_OutVelocity[i], m_CouplerIndexInDest);
+      }
     }
   }
 }
 
+void GOCoupler::RefreshState() {
+  bool on = IsActive();
+
+  if (m_UnisonOff)
+    r_OrganModel.GetManual(m_SourceManual)->SetUnisonOff(on);
+  else {
+    GOManual *dest = r_OrganModel.GetManual(m_DestinationManual);
+
+    for (unsigned l = m_OutVelocity.size(), i = 0; i < l; i++)
+      // check if the key is set. Otherwise propagating is not necessary
+      if (m_OutVelocity[i])
+        dest->PropagateKeyToCouplers(i);
+  }
+}
+
 bool GOCoupler::IsIntermanual() {
   return m_SourceManual != m_DestinationManual;
 }
