@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -56,6 +56,12 @@ private:
 
 public:
   GOCoupler(GOOrganModel &organModel, unsigned sourceManual);
+
+  bool IsRecursive() const {
+    return m_CoupleToSubsequentUnisonIntermanualCouplers;
+  }
+  void SetRecursive(bool isRecursive);
+
   void Init(
     GOConfigReader &cfg,
     wxString group,
@@ -66,6 +72,10 @@ public:
     int dest_manual,
     GOCouplerType coupler_type);
   void Load(GOConfigReader &cfg, wxString group);
+
+  // send key states for all chained couplers
+  void RefreshState();
+
   void SetKey(
     unsigned note,
     const std::vector<unsigned> &velocities,
