@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -116,8 +116,9 @@ public:
   void Load(GOConfigReader &cfg, const wxString &group, int manualNumber);
   void LoadDivisionals(GOConfigReader &cfg);
   unsigned RegisterCoupler(GOCoupler *coupler);
-  void SetKey(
-    unsigned note, unsigned velocity, GOCoupler *prev, unsigned couplerID);
+  // send the key state to all outgoing couplers
+  void PropagateKeyToCouplers(unsigned note);
+  void SetKey(unsigned note, unsigned velocity, unsigned couplerID);
   void Set(unsigned note, unsigned velocity);
   void SetUnisonOff(bool on);
   void Update();
@@ -152,7 +153,7 @@ public:
    */
   int FindCouplerByName(const wxString &name) const;
   void AddCoupler(GOCoupler *coupler);
-  unsigned GetDivisionalCount();
+  unsigned GetDivisionalCount() const { return m_divisionals.size(); }
   GODivisionalButtonControl *GetDivisional(unsigned index);
   void AddDivisional(GODivisionalButtonControl *divisional);
   unsigned GetTremulantCount() const { return m_tremulant_ids.size(); }
