@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -27,8 +27,7 @@
 GOSoundingPipe::GOSoundingPipe(
   GOOrganModel *pOrganModel,
   GORank *rank,
-  bool percussive,
-  int sampler_group_id,
+  unsigned windchestN,
   unsigned midi_key_number,
   unsigned harmonic_number,
   float min_volume,
@@ -36,20 +35,17 @@ GOSoundingPipe::GOSoundingPipe(
   bool retune)
   : GOPipe(pOrganModel, rank, midi_key_number),
     p_OrganModel(pOrganModel),
-    m_Sampler(NULL),
+    p_CurrentLoopSampler(nullptr),
+    m_LastStart(0),
     m_LastStop(0),
     m_Instances(0),
-    m_Tremulant(false),
-    m_AttackInfo(),
-    m_ReleaseInfo(),
+    m_AttackFileInfos(),
+    m_ReleaseFileInfos(),
     m_Filename(),
-    m_SamplerGroupID(sampler_group_id),
+    m_WindchestN(windchestN),
     m_AudioGroupID(0),
-    m_Percussive(percussive),
     m_TemperamentOffset(0),
     m_HarmonicNumber(harmonic_number),
-    m_LoopCrossfadeLength(0),
-    m_ReleaseCrossfadeLength(0),
     m_MinVolume(min_volume),
     m_MaxVolume(max_volume),
     m_OdfMidiKeyNumber(-1),
@@ -58,19 +54,58 @@ GOSoundingPipe::GOSoundingPipe(
     m_SampleMidiPitchFraction(0.0),
     m_RetunePipe(retune),
     m_IsTemperamentOriginalBased(true),
+    m_SoundProvider(this),
     m_PipeConfigNode(
-      &rank->GetPipeConfig(), pOrganModel, this, &m_SoundProvider) {}
+      &rank->GetPipeConfig(), *pOrganModel, this, &m_SoundProvider) {}
+
+void GOSoundingPipe::Init(
+  GOConfigReader &cfg,
+  const wxString &group,
+  const wxString &prefix,
+  const wxString &filename) {
+  SetGroupAndPrefix(group, prefix);
+  p_OrganModel->RegisterCacheObject(this);
+  m_Filename = filename;
+  m_PipeConfigNode.Init(cfg, group, prefix);
+  m_OdfMidiKeyNumber = -1;
+  m_OdfMidiPitchFraction = -1.0;
+  UpdateAmplitude();
+  p_OrganModel->GetWindchest(m_WindchestN - 1)->AddPipe(this);
+
+  GOSoundProviderWave::AttackFileInfo ainfo;
+
+  ainfo.filename.AssignResource(m_Filename);
+  ainfo.m_WaveTremulantStateFor = BOOL3_DEFAULT;
+  ainfo.percussive = m_PipeConfigNode.GetEffectivePercussive();
+  ainfo.load_release = !ainfo.percussive;
+  ainfo.max_playback_time = -1;
+  ainfo.cue_point = -1;
+  ainfo.min_attack_velocity = 0;
+  ainfo.max_released_time = -1;
+  ainfo.attack_start = 0;
+  ainfo.release_end = -1;
+  ainfo.m_LoopCrossfadeLength = 0;
+  ainfo.m_ReleaseCrossfadeLength = 0;
+  m_AttackFileInfos.push_back(ainfo);
+
+  m_SoundProvider.SetVelocityParameter(m_MinVolume, m_MaxVolume);
+  m_PipeConfigNode.SetName(
+    wxString::Format(_("%d: %s"), m_MidiKeyNumber, m_Filename.c_str()));
+}
+
+void GOSoundingPipe::LoadAttackFileInfo(
+  GOConfigReader &cfg,
+  const wxString &group,
+  const wxString &prefix,
+  bool isMainForSecondaryAttacks) {
+  GOSoundProviderWave::AttackFileInfo ainfo;
 
-void GOSoundingPipe::LoadAttack(
-  GOConfigReader &cfg, wxString group, wxString prefix) {
-  attack_load_info ainfo;
   ainfo.filename.Assign(cfg.ReadFileName(ODFSetting, group, prefix));
-  ainfo.sample_group = cfg.ReadInteger(
-    ODFSetting, group, prefix + wxT("IsTremulant"), -1, 1, false, -1);
+  ainfo.m_WaveTremulantStateFor = cfg.ReadBool3FromInt(
+    ODFSetting, group, prefix + wxT("IsTremulant"), false);
+  ainfo.percussive = m_PipeConfigNode.GetEffectivePercussive();
   ainfo.load_release = cfg.ReadBoolean(
-    ODFSetting, group, prefix + wxT("LoadRelease"), false, !m_Percussive);
-  ;
-  ainfo.percussive = m_Percussive;
+    ODFSetting, group, prefix + wxT("LoadRelease"), false, !ainfo.percussive);
   ainfo.max_playback_time = cfg.ReadInteger(
     ODFSetting, group, prefix + wxT("MaxKeyPressTime"), -1, 100000, false, -1);
   ainfo.cue_point = cfg.ReadInteger(
@@ -130,42 +165,61 @@ void GOSoundingPipe::LoadAttack(
       true);
     ainfo.loops.push_back(linfo);
   }
+  ainfo.m_LoopCrossfadeLength = cfg.ReadInteger(
+    ODFSetting, group, prefix + wxT("LoopCrossfadeLength"), 0, 3000, false, 0);
+  ainfo.m_ReleaseCrossfadeLength
+    = isMainForSecondaryAttacks || ainfo.load_release ? cfg.ReadInteger(
+        ODFSetting,
+        group,
+        prefix + wxT("ReleaseCrossfadeLength"),
+        0,
+        3000,
+        false,
+        0)
+                                                      : 0;
 
-  m_AttackInfo.push_back(ainfo);
+  m_AttackFileInfos.push_back(ainfo);
 }
 
-void GOSoundingPipe::Init(
-  GOConfigReader &cfg, wxString group, wxString prefix, wxString filename) {
-  p_OrganModel->RegisterCacheObject(this);
-  m_Filename = filename;
-  m_PipeConfigNode.Init(cfg, group, prefix);
-  m_OdfMidiKeyNumber = -1;
-  m_OdfMidiPitchFraction = -1.0;
-  m_LoopCrossfadeLength = 0;
-  m_ReleaseCrossfadeLength = 0;
-  UpdateAmplitude();
-  p_OrganModel->GetWindchest(m_SamplerGroupID - 1)->AddPipe(this);
+void GOSoundingPipe::LoadReleaseFileInfo(
+  GOConfigReader &cfg, const wxString &group, const wxString &prefix) {
+  GOSoundProviderWave::ReleaseFileInfo rinfo;
 
-  attack_load_info ainfo;
-  ainfo.filename.AssignResource(m_Filename);
-  ainfo.sample_group = -1;
-  ainfo.load_release = !m_Percussive;
-  ainfo.percussive = m_Percussive;
-  ainfo.max_playback_time = -1;
-  ainfo.cue_point = -1;
-  ainfo.min_attack_velocity = 0;
-  ainfo.max_released_time = -1;
-  ainfo.attack_start = 0;
-  ainfo.release_end = -1;
-  m_AttackInfo.push_back(ainfo);
-
-  m_SoundProvider.SetVelocityParameter(m_MinVolume, m_MaxVolume);
-  m_PipeConfigNode.SetName(
-    wxString::Format(_("%d: %s"), m_MidiKeyNumber, m_Filename.c_str()));
+  rinfo.filename.Assign(cfg.ReadFileName(ODFSetting, group, prefix));
+  rinfo.m_WaveTremulantStateFor = cfg.ReadBool3FromInt(
+    ODFSetting, group, prefix + wxT("IsTremulant"), false);
+  rinfo.max_playback_time = cfg.ReadInteger(
+    ODFSetting, group, prefix + wxT("MaxKeyPressTime"), -1, 100000, false, -1);
+  rinfo.cue_point = cfg.ReadInteger(
+    ODFSetting,
+    group,
+    prefix + wxT("CuePoint"),
+    -1,
+    MAX_SAMPLE_LENGTH,
+    false,
+    -1);
+  rinfo.release_end = cfg.ReadInteger(
+    ODFSetting,
+    group,
+    prefix + wxT("ReleaseEnd"),
+    -1,
+    MAX_SAMPLE_LENGTH,
+    false,
+    -1);
+  rinfo.m_ReleaseCrossfadeLength = cfg.ReadInteger(
+    ODFSetting,
+    group,
+    prefix + wxT("ReleaseCrossfadeLength"),
+    0,
+    3000,
+    false,
+    0);
+  m_ReleaseFileInfos.push_back(rinfo);
 }
 
 void GOSoundingPipe::Load(
-  GOConfigReader &cfg, wxString group, wxString prefix) {
+  GOConfigReader &cfg, const wxString &group, const wxString &prefix) {
+  SetGroupAndPrefix(group, prefix);
   p_OrganModel->RegisterCacheObject(this);
   m_Filename = cfg.ReadStringTrim(ODFSetting, group, prefix);
   m_PipeConfigNode.Load(cfg, group, prefix);
@@ -177,16 +231,14 @@ void GOSoundingPipe::Load(
     1024,
     false,
     m_HarmonicNumber);
-  m_SamplerGroupID = cfg.ReadInteger(
+  m_WindchestN = cfg.ReadInteger(
     ODFSetting,
     group,
     prefix + wxT("WindchestGroup"),
     1,
-    p_OrganModel->GetWindchestGroupCount(),
+    p_OrganModel->GetWindchestCount(),
     false,
-    m_SamplerGroupID);
-  m_Percussive = cfg.ReadBoolean(
-    ODFSetting, group, prefix + wxT("Percussive"), false, m_Percussive);
+    m_WindchestN);
   m_OdfMidiKeyNumber = cfg.ReadInteger(
     ODFSetting, group, prefix + wxT("MIDIKeyNumber"), -1, 127, false, -1);
   m_OdfMidiPitchFraction = cfg.ReadFloat(
@@ -197,52 +249,33 @@ void GOSoundingPipe::Load(
     100.0,
     false,
     -1.0);
-  m_LoopCrossfadeLength = cfg.ReadInteger(
-    ODFSetting, group, prefix + wxT("LoopCrossfadeLength"), 0, 3000, false, 0);
-  m_ReleaseCrossfadeLength = cfg.ReadInteger(
-    ODFSetting,
-    group,
-    prefix + wxT("ReleaseCrossfadeLength"),
-    0,
-    3000,
-    false,
-    0);
   m_RetunePipe = cfg.ReadBoolean(
     ODFSetting, group, prefix + wxT("AcceptsRetuning"), false, m_RetunePipe);
   UpdateAmplitude();
-  p_OrganModel->GetWindchest(m_SamplerGroupID - 1)->AddPipe(this);
-
-  LoadAttack(cfg, group, prefix);
+  p_OrganModel->GetWindchest(m_WindchestN - 1)->AddPipe(this);
 
   unsigned attack_count = cfg.ReadInteger(
     ODFSetting, group, prefix + wxT("AttackCount"), 0, 100, false, 0);
-  for (unsigned i = 0; i < attack_count; i++)
-    LoadAttack(cfg, group, prefix + wxString::Format(wxT("Attack%03d"), i + 1));
+
+  LoadAttackFileInfo(cfg, group, prefix, attack_count > 0);
+
+  for (unsigned i = 1; i <= attack_count; i++)
+    LoadAttackFileInfo(
+      cfg, group, wxString::Format(wxT("%sAttack%03d"), prefix, i), false);
 
   unsigned release_count = cfg.ReadInteger(
     ODFSetting, group, prefix + wxT("ReleaseCount"), 0, 100, false, 0);
-  for (unsigned i = 0; i < release_count; i++) {
-    release_load_info rinfo;
-    wxString p = prefix + wxString::Format(wxT("Release%03d"), i + 1);
 
-    rinfo.filename.Assign(cfg.ReadFileName(ODFSetting, group, p));
-    rinfo.sample_group = cfg.ReadInteger(
-      ODFSetting, group, p + wxT("IsTremulant"), -1, 1, false, -1);
-    rinfo.max_playback_time = cfg.ReadInteger(
-      ODFSetting, group, p + wxT("MaxKeyPressTime"), -1, 100000, false, -1);
-    rinfo.cue_point = cfg.ReadInteger(
-      ODFSetting, group, p + wxT("CuePoint"), -1, MAX_SAMPLE_LENGTH, false, -1);
-    rinfo.release_end = cfg.ReadInteger(
-      ODFSetting,
+  if (m_PipeConfigNode.IsEffectiveIndependentRelease() && !release_count)
+    wxLogWarning(
+      _("The pipe %s/%s has independent release but %sReleaseCount=0"),
       group,
-      p + wxT("ReleaseEnd"),
-      -1,
-      MAX_SAMPLE_LENGTH,
-      false,
-      -1);
+      prefix,
+      prefix);
 
-    m_ReleaseInfo.push_back(rinfo);
-  }
+  for (unsigned i = 1; i <= release_count; i++)
+    LoadReleaseFileInfo(
+      cfg, group, wxString::Format(wxT("%sRelease%03d"), prefix, i));
 
   m_MinVolume = cfg.ReadFloat(
     ODFSetting, group, wxT("MinVelocityVolume"), 0, 1000, false, m_MinVolume);
@@ -256,33 +289,25 @@ void GOSoundingPipe::Load(
 void GOSoundingPipe::LoadData(
   const GOFileStore &fileStore, GOMemoryPool &pool) {
   try {
-    m_SoundProvider.LoadFromFile(
+    m_SoundProvider.LoadFromMultipleFiles(
       fileStore,
       pool,
-      m_AttackInfo,
-      m_ReleaseInfo,
+      m_AttackFileInfos,
+      m_ReleaseFileInfos,
       m_PipeConfigNode.GetEffectiveBitsPerSample(),
       m_PipeConfigNode.GetEffectiveChannels(),
       m_PipeConfigNode.GetEffectiveCompress(),
-      (loop_load_type)m_PipeConfigNode.GetEffectiveLoopLoad(),
+      (GOSoundProviderWave::LoopLoadType)
+        m_PipeConfigNode.GetEffectiveLoopLoad(),
       m_PipeConfigNode.GetEffectiveAttackLoad(),
-      m_PipeConfigNode.GetEffectiveReleaseLoad(),
-      m_LoopCrossfadeLength,
-      m_ReleaseCrossfadeLength);
+      m_PipeConfigNode.GetEffectiveReleaseLoad());
     Validate();
-  } catch (wxString str) {
-    m_SoundProvider.ClearData();
-    throw wxString::Format(
-      _("Error while loading samples for rank %s pipe %s: %s"),
-      m_Rank->GetName().c_str(),
-      GetLoadTitle().c_str(),
-      str.c_str());
   } catch (std::bad_alloc &ba) {
     m_SoundProvider.ClearData();
     throw GOOutOfMemory();
-  } catch (GOOutOfMemory e) {
+  } catch (...) {
     m_SoundProvider.ClearData();
-    throw GOOutOfMemory();
+    throw;
   }
 }
 
@@ -301,11 +326,11 @@ bool GOSoundingPipe::LoadCache(GOMemoryPool &pool, GOCache &cache) {
   }
 }
 
-bool GOSoundingPipe::SaveCache(GOCacheWriter &cache) {
+bool GOSoundingPipe::SaveCache(GOCacheWriter &cache) const {
   return m_SoundProvider.SaveCache(cache);
 }
 
-void GOSoundingPipe::UpdateHash(GOHash &hash) {
+void GOSoundingPipe::UpdateHash(GOHash &hash) const {
   hash.Update(m_Filename);
   hash.Update(m_PipeConfigNode.GetEffectiveBitsPerSample());
   hash.Update(m_PipeConfigNode.GetEffectiveCompress());
@@ -314,33 +339,35 @@ void GOSoundingPipe::UpdateHash(GOHash &hash) {
   hash.Update(m_PipeConfigNode.GetEffectiveAttackLoad());
   hash.Update(m_PipeConfigNode.GetEffectiveReleaseLoad());
   hash.Update(m_OdfMidiKeyNumber);
-  hash.Update(m_LoopCrossfadeLength);
-  hash.Update(m_ReleaseCrossfadeLength);
+  hash.Update(m_PipeConfigNode.IsEffectiveIndependentRelease());
 
-  hash.Update(m_AttackInfo.size());
-  for (unsigned i = 0; i < m_AttackInfo.size(); i++) {
-    m_AttackInfo[i].filename.Hash(hash);
-    hash.Update(m_AttackInfo[i].sample_group);
-    hash.Update(m_AttackInfo[i].max_playback_time);
-    hash.Update(m_AttackInfo[i].load_release);
-    hash.Update(m_AttackInfo[i].percussive);
-    hash.Update(m_AttackInfo[i].cue_point);
-    hash.Update(m_AttackInfo[i].loops.size());
-    hash.Update(m_AttackInfo[i].attack_start);
-    hash.Update(m_AttackInfo[i].release_end);
-    for (unsigned j = 0; j < m_AttackInfo[i].loops.size(); j++) {
-      hash.Update(m_AttackInfo[i].loops[j].m_StartPosition);
-      hash.Update(m_AttackInfo[i].loops[j].m_EndPosition);
+  hash.Update(m_AttackFileInfos.size());
+  for (const auto &a : m_AttackFileInfos) {
+    a.filename.Hash(hash);
+    hash.Update(a.m_WaveTremulantStateFor);
+    hash.Update(a.max_playback_time);
+    hash.Update(a.load_release);
+    hash.Update(a.percussive);
+    hash.Update(a.cue_point);
+    hash.Update(a.loops.size());
+    hash.Update(a.attack_start);
+    hash.Update(a.release_end);
+    for (unsigned j = 0; j < a.loops.size(); j++) {
+      hash.Update(a.loops[j].m_StartPosition);
+      hash.Update(a.loops[j].m_EndPosition);
     }
+    hash.Update(a.m_LoopCrossfadeLength);
+    hash.Update(a.m_ReleaseCrossfadeLength);
   }
 
-  hash.Update(m_ReleaseInfo.size());
-  for (unsigned i = 0; i < m_ReleaseInfo.size(); i++) {
-    m_ReleaseInfo[i].filename.Hash(hash);
-    hash.Update(m_ReleaseInfo[i].sample_group);
-    hash.Update(m_ReleaseInfo[i].max_playback_time);
-    hash.Update(m_ReleaseInfo[i].cue_point);
-    hash.Update(m_ReleaseInfo[i].release_end);
+  hash.Update(m_ReleaseFileInfos.size());
+  for (const auto &r : m_ReleaseFileInfos) {
+    r.filename.Hash(hash);
+    hash.Update(r.m_WaveTremulantStateFor);
+    hash.Update(r.max_playback_time);
+    hash.Update(r.cue_point);
+    hash.Update(r.release_end);
+    hash.Update(r.m_ReleaseCrossfadeLength);
   }
 }
 
@@ -384,28 +411,30 @@ void GOSoundingPipe::Validate() {
   if (!m_PipeConfigNode.GetEffectiveChannels())
     return;
 
-  if (m_SoundProvider.checkForMissingAttack()) {
+  if (m_SoundProvider.CheckForMissingAttack()) {
     wxLogWarning(
       _("rank %s pipe %s: attack with MaxTimeSinceLastRelease=-1 missing"),
       m_Rank->GetName().c_str(),
       GetLoadTitle().c_str());
   }
 
-  if (m_SoundProvider.checkForMissingRelease()) {
+  if (m_SoundProvider.CheckForMissingRelease()) {
     wxLogWarning(
       _("rank %s pipe %s: default release is missing"),
       m_Rank->GetName().c_str(),
       GetLoadTitle().c_str());
   }
 
-  if (m_SoundProvider.checkMissingRelease()) {
+  if (m_SoundProvider.CheckMissingRelease()) {
     wxLogWarning(
       _("rank %s pipe %s: no release defined"),
       m_Rank->GetName().c_str(),
       GetLoadTitle().c_str());
   }
 
-  if (m_SoundProvider.checkNotNecessaryRelease()) {
+  if (
+    !m_PipeConfigNode.IsEffectiveIndependentRelease()
+    && m_SoundProvider.CheckNotNecessaryRelease()) {
     wxLogWarning(
       _("rank %s pipe %s: percussive sample with a release"),
       m_Rank->GetName().c_str(),
@@ -444,15 +473,17 @@ void GOSoundingPipe::Validate() {
   }
 }
 
-void GOSoundingPipe::SetTremulant(bool on) {
-  if (on != m_Tremulant) {
-    m_Tremulant = on;
-    m_SoundProvider.UseSampleGroup((unsigned)on);
+void GOSoundingPipe::SetWaveTremulant(bool on) {
+  if (m_SoundProvider.IsWaveTremulant() != on) {
+    m_SoundProvider.SetWaveTremulant(on);
 
     GOSoundEngine *pSoundEngine = GetSoundEngine();
 
-    if (pSoundEngine && m_Sampler)
-      pSoundEngine->SwitchSample(&m_SoundProvider, m_Sampler);
+    if (
+      pSoundEngine && p_CurrentLoopSampler
+      && !m_SoundProvider.IsWaveTremulantStateSuitable(
+        p_CurrentLoopSampler->m_WaveTremulantStateFor))
+      pSoundEngine->SwitchSample(&m_SoundProvider, p_CurrentLoopSampler);
   }
 }
 
@@ -462,31 +493,45 @@ void GOSoundingPipe::VelocityChanged(
 
   if (!m_Instances && velocity) {
     // the key pressed
-    m_Sampler = pSoundEngine ? pSoundEngine->StartSample(
-                  &m_SoundProvider,
-                  m_SamplerGroupID,
-                  m_AudioGroupID,
-                  velocity,
-                  m_PipeConfigNode.GetEffectiveDelay(),
-                  m_LastStop)
-                             : nullptr;
-    if (m_Sampler)
+    GOSoundSampler *pSampler = pSoundEngine ? pSoundEngine->StartPipeSample(
+                                 &m_SoundProvider,
+                                 m_WindchestN,
+                                 m_AudioGroupID,
+                                 velocity,
+                                 m_PipeConfigNode.GetEffectiveDelay(),
+                                 m_LastStop,
+                                 false,
+                                 &m_LastStart)
+                                            : nullptr;
+    if (pSampler) {
       m_Instances++;
-    if (m_SoundProvider.IsOneshot())
-      m_Sampler = nullptr;
+      if (!m_SoundProvider.IsOneshot()) {
+        p_CurrentLoopSampler = pSampler;
+      }
+    }
   } else if (m_Instances && !velocity) {
     // the key released
     m_Instances--;
-    if (m_Sampler) {
+    if (p_CurrentLoopSampler) {
       m_LastStop = pSoundEngine
-        ? pSoundEngine->StopSample(&m_SoundProvider, m_Sampler)
+        ? pSoundEngine->StopSample(&m_SoundProvider, p_CurrentLoopSampler)
         : 0;
-      m_Sampler = nullptr;
-    }
-  } else if (m_Sampler && last_velocity != velocity)
+      p_CurrentLoopSampler = nullptr;
+    } else if (m_PipeConfigNode.IsEffectiveIndependentRelease() && pSoundEngine)
+      pSoundEngine->StartPipeSample(
+        &m_SoundProvider,
+        m_WindchestN,
+        m_AudioGroupID,
+        last_velocity,
+        m_PipeConfigNode.GetEffectiveDelay(),
+        m_LastStart,
+        true,
+        &m_LastStop);
+  } else if (p_CurrentLoopSampler && last_velocity != velocity)
     // the key was pressed before and the velocity is changed now
     if (pSoundEngine)
-      pSoundEngine->UpdateVelocity(&m_SoundProvider, m_Sampler, velocity);
+      pSoundEngine->UpdateVelocity(
+        &m_SoundProvider, p_CurrentLoopSampler, velocity);
 }
 
 void GOSoundingPipe::UpdateAmplitude() {
@@ -528,9 +573,8 @@ void GOSoundingPipe::PreparePlayback() {
 
 void GOSoundingPipe::AbortPlayback() {
   m_Instances = 0;
-  m_Tremulant = false;
-  m_Sampler = 0;
+  p_CurrentLoopSampler = 0;
   m_LastStop = 0;
-  m_SoundProvider.UseSampleGroup(0);
+  m_SoundProvider.SetWaveTremulant(false);
   GOPipe::AbortPlayback();
 }
