@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -13,20 +13,10 @@ GOPipeConfigTreeNode::GOPipeConfigTreeNode(
   GOPipeConfigNode *parent,
   GOOrganModel *organModel,
   GOPipeUpdateCallback *callback)
-  : GOPipeConfigNode(parent, organModel, this, NULL),
+  : GOPipeConfigNode(parent, *organModel, this, NULL),
     m_Childs(),
     m_Callback(callback) {}
 
-void GOPipeConfigTreeNode::AddChild(GOPipeConfigNode *node) {
-  m_Childs.push_back(node);
-}
-
-unsigned GOPipeConfigTreeNode::GetChildCount() { return m_Childs.size(); }
-
-GOPipeConfigNode *GOPipeConfigTreeNode::GetChild(unsigned index) {
-  return m_Childs[index];
-}
-
 void GOPipeConfigTreeNode::UpdateAmplitude() {
   for (unsigned i = 0; i < m_Childs.size(); i++)
     m_Childs[i]->GetPipeConfig().GetCallback()->UpdateAmplitude();
@@ -55,7 +45,7 @@ void GOPipeConfigTreeNode::UpdateReleaseTail() {
     m_Callback->UpdateReleaseTail();
 }
 
-GOSampleStatistic GOPipeConfigTreeNode::GetStatistic() {
+GOSampleStatistic GOPipeConfigTreeNode::GetStatistic() const {
   GOSampleStatistic stat = GOPipeConfigNode::GetStatistic();
   for (unsigned i = 0; i < m_Childs.size(); i++)
     stat.Cumulate(m_Childs[i]->GetStatistic());
