@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -8,62 +8,75 @@
 #ifndef GOSOUNDPROVIDER_H_
 #define GOSOUNDPROVIDER_H_
 
+#include <cstdint>
 #include <vector>
 
-#include "GOStatisticCallback.h"
 #include "ptrvector.h"
 
-class GOAudioSection;
+#include "GOBool3.h"
+#include "GOStatisticCallback.h"
+
+class GOSoundAudioSection;
 class GOCache;
 class GOCacheWriter;
+class GOHash;
 class GOMemoryPool;
 
 typedef struct audio_section_stream_s audio_section_stream;
 
-typedef struct {
-  int sample_group;
-  unsigned min_attack_velocity;
-  unsigned max_released_time;
-} attack_section_info;
-
-typedef struct {
-  int sample_group;
-  unsigned max_playback_time;
-} release_section_info;
-
 class GOSoundProvider : public GOStatisticCallback {
 protected:
+  struct AttackSelector {
+    unsigned min_attack_velocity;
+    unsigned max_released_time;
+    GOBool3 m_WaveTremulantStateFor;
+  };
+
+  struct ReleaseSelector {
+    unsigned max_playback_time;
+    GOBool3 m_WaveTremulantStateFor;
+  };
+
   unsigned m_MidiKeyNumber;
   float m_MidiPitchFract;
   float m_Gain;
   float m_Tuning;
-  bool m_SampleGroup;
+  bool m_IsWaveTremulantActive;
   unsigned m_ReleaseTail;
-  ptr_vector<GOAudioSection> m_Attack;
-  std::vector<attack_section_info> m_AttackInfo;
-  ptr_vector<GOAudioSection> m_Release;
-  std::vector<release_section_info> m_ReleaseInfo;
+  ptr_vector<GOSoundAudioSection> m_Attack;
+  std::vector<AttackSelector> m_AttackInfo;
+  ptr_vector<GOSoundAudioSection> m_Release;
+  std::vector<ReleaseSelector> m_ReleaseInfo;
   void ComputeReleaseAlignmentInfo();
   float m_VelocityVolumeBase;
   float m_VelocityVolumeIncrement;
-  unsigned m_ReleaseCrossfadeLength;
+  unsigned m_AttackSwitchCrossfadeLength;
 
 public:
+  static void UpdateCacheHash(GOHash &hash);
+
   GOSoundProvider();
   virtual ~GOSoundProvider();
 
   void ClearData();
 
   virtual bool LoadCache(GOMemoryPool &pool, GOCache &cache);
-  virtual bool SaveCache(GOCacheWriter &cache);
+  virtual bool SaveCache(GOCacheWriter &cache) const;
+
+  bool IsWaveTremulant() const { return m_IsWaveTremulantActive; }
+  void SetWaveTremulant(bool isActive) { m_IsWaveTremulantActive = isActive; }
 
-  void UseSampleGroup(unsigned sample_group);
   void SetVelocityParameter(float min_volume, float max_volume);
 
-  const GOAudioSection *GetRelease(
-    const audio_section_stream *handle, double playback_time) const;
-  const GOAudioSection *GetAttack(
-    unsigned velocity, unsigned released_time) const;
+  bool IsWaveTremulantStateSuitable(GOBool3 waveTremulantStateFor) const {
+    return to_bool(waveTremulantStateFor, m_IsWaveTremulantActive)
+      == m_IsWaveTremulantActive;
+  }
+
+  const GOSoundAudioSection *GetAttack(
+    unsigned velocity, unsigned releasedDurationMs) const;
+  const GOSoundAudioSection *GetRelease(
+    GOBool3 waveTremulantStateFor, unsigned playbackDurationMs) const;
   float GetGain() const;
   int IsOneshot() const;
 
@@ -74,14 +87,16 @@ public:
 
   unsigned GetMidiKeyNumber() const;
   float GetMidiPitchFract() const;
-  unsigned GetReleaseCrossfadeLength() const;
+  unsigned GetAttackSwitchCrossfadeLength() const {
+    return m_AttackSwitchCrossfadeLength;
+  }
 
   float GetVelocityVolume(unsigned velocity) const;
 
-  bool checkForMissingAttack();
-  bool checkForMissingRelease();
-  bool checkMissingRelease();
-  bool checkNotNecessaryRelease();
+  bool CheckForMissingAttack();
+  bool CheckForMissingRelease();
+  bool CheckMissingRelease();
+  bool CheckNotNecessaryRelease();
 
   GOSampleStatistic GetStatistic();
 };
