@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -8,7 +8,7 @@
 #ifndef GOSOUNDRELEASEALIGNTABLE_H
 #define GOSOUNDRELEASEALIGNTABLE_H
 
-class GOAudioSection;
+class GOSoundAudioSection;
 class GOCache;
 class GOCacheWriter;
 typedef struct audio_section_stream_s audio_section_stream;
@@ -31,7 +31,7 @@ public:
   bool Save(GOCacheWriter &cache);
 
   void ComputeTable(
-    const GOAudioSection &m_release,
+    const GOSoundAudioSection &m_release,
     int phase_align_max_amplitude,
     int phase_align_max_derivative,
     unsigned int sample_rate,
