@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2024 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -14,13 +14,12 @@
 class GOSoundProvider;
 class GOSoundWindchestWorkItem;
 
-class GOSoundSampler {
-public:
+struct GOSoundSampler {
   GOSoundSampler *next;
-  const GOSoundProvider *pipe;
-  int sampler_group_id;
-  GOSoundWindchestWorkItem *windchest;
-  unsigned audio_group_id;
+  const GOSoundProvider *p_SoundProvider;
+  int m_SamplerTaskId;
+  GOSoundWindchestWorkItem *p_WindchestTask;
+  unsigned m_AudioGroupId;
   audio_section_stream stream;
   GOSoundFader fader;
   uint64_t time;
@@ -29,6 +28,7 @@ public:
   /* current index of the current block into this sample */
   volatile unsigned long stop;
   volatile unsigned long new_attack;
+  GOBool3 m_WaveTremulantStateFor;
   bool is_release;
   unsigned drop_counter;
 };
