@@ -15,7 +15,7 @@
 class GOSoundRtPort : public GOSoundPort {
 private:
   RtAudio *m_rtApi;
-  unsigned m_RtDevIndex;
+  unsigned m_RtDevId;
   unsigned m_nBuffers;
 
   static int Callback(
@@ -28,6 +28,8 @@ private:
 
   static wxString getName(RtAudio *rtApi, const RtAudio::DeviceInfo &devInfo);
 
+  bool processRtResult(RtAudioErrorType rtResult, bool isToThrowOnError = true);
+
   GOSoundRtPort(
     GOSound *sound, RtAudio *rtApi, unsigned rtDevIndex, const wxString &name);
 
