@@ -12,9 +12,14 @@
 #include "GOSoundScheduler.h"
 #include "sound/scheduler/GOSoundWorkItem.h"
 #include "threading/GOMutexLocker.h"
+#include <unistd.h>
 
 GOSoundThread::GOSoundThread(GOSoundScheduler *scheduler)
-  : GOThread(), m_Scheduler(scheduler), m_Condition(m_Mutex) {
+  : GOThread(),
+    m_Scheduler(scheduler),
+    m_Condition(m_Mutex),
+    m_IdleStateReachedCondition(m_Mutex),
+    m_IsIdle(false) {
   wxLogDebug(wxT("Create Thread"));
 }
 
@@ -35,15 +40,25 @@ void GOSoundThread::Entry() {
       break;
 
     GOMutexLocker lock(m_Mutex, false, "GOSoundThread::Entry", this);
-
     if (!lock.IsLocked() || ShouldStop())
       break;
+    m_IsIdle = true;
+    m_IdleStateReachedCondition.Broadcast();
     if (!m_Condition.WaitOrStop("GOSoundThread::Entry"))
       break;
+    m_IsIdle = false;
   }
+
   return;
 }
 
+void GOSoundThread::WaitForIdle() {
+  GOMutexLocker lock(m_Mutex, false, "GOSoundThread::WaitForIdle");
+  while (!m_IsIdle) {
+    m_IdleStateReachedCondition.Wait();
+  }
+}
+
 void GOSoundThread::Run() { Start(); }
 
 void GOSoundThread::Wakeup() { m_Condition.Signal(); }
